from typing import List

from blue_options.terminal import show_usage


def help_browse(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@browse",
            "<url>",
            "[<description>]",
        ],
        "browse <url>.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_watch(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("~clear,dryrun,", mono=mono),
            "seconds=<seconds>",
        ]
    )

    return show_usage(
        [
            "@watch",
            f"[{options}]",
            "<command-line>",
        ],
        "watch <command-line>.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_build_README(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "abcli",
) -> str:
    options = "push"

    callable = f"{plugin_name} build_README"

    if plugin_name == "abcli":
        options = f"{options},plugin=<plugin-name>"
        callable = "@build_README"

    return show_usage(
        callable.split(" ")
        + [
            f"[{options}]",
        ],
        f"build {plugin_name}/README.md.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_cat(
    tokens: List[str],
    mono: bool,
) -> str:
    return "\n".join(
        [
            show_usage(
                [
                    "@env",
                    "dot",
                    "cat",
                    "[ | <env-name> | config | sample ]",
                ],
                "cat env.",
                mono=mono,
            ),
            show_usage(
                [
                    "@env",
                    "dot",
                    "cat",
                    "jetson_nano | rpi",
                    "<machine-name>",
                ],
                "cat .env from machine.",
                mono=mono,
            ),
        ]
    )


def help_cp(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@env",
            "dot",
            "cp | copy",
            "<env-name>",
            "local | jetson_nano | rpi",
            "[<machine-name>]",
        ],
        "cp <env-name> to machine.",
        mono=mono,
    )


def help_edit(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@env",
            "dot",
            "edit",
            "jetson_nano | rpi",
            "<machine-name>",
        ],
        "edit .env on machine.",
        mono=mono,
    )


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@env",
            "dot",
            "get",
            "<variable>",
        ],
        "<variable>.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@env",
            "dot",
            "list",
        ],
        "list env repo.",
        mono=mono,
    )


def help_load(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "caller,filename=<.env>,plugin=<plugin-name>,ssm,suffix=/tests,verbose"

    return show_usage(
        [
            "@env",
            "dot",
            "load",
            f"[{options}]",
        ],
        "load .env.",
        mono=mono,
    )


def help_set(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@env",
            "dot",
            "set",
            "<variable>",
            "<value>",
        ],
        "<variable> = <value>.",
        mono=mono,
    )


help_functions = {
    "cat": help_cat,
    "cp": help_cp,
    "edit": help_edit,
    "get": help_get,
    "list": help_list,
    "load": help_load,
    "set": help_set,
}



from typing import List

from blue_options.terminal import show_usage, xtra


def help_(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "clipboard|filename=<filename>|key|screen,env=<env-name>,eval,plugin=<plugin-name>,~log"

    return show_usage(
        [
            "@seed",
            "<target>",
            f"[{options}]",
            "[cat,dryrun]",
            "<command-line>",
        ],
        "generate and output a seed 🌱.",
        mono=mono,
    )


def help_eject(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@seed",
            "eject",
        ],
        "eject seed 🌱.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@seed",
            "list",
        ],
        "list seed 🌱 targets.",
        mono=mono,
    )


help_functions = {
    "": help_,
    "eject": help_eject,
    "list": help_list,
}



from typing import List

from blue_options.terminal import show_usage


def help_badge(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@badge",
            'clear | "🪄"',
        ],
        "update badge.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    return "\n".join(
        [
            show_usage(
                [
                    "@ssm",
                    "get",
                    "<secret-name>",
                ],
                "get <secret-name>.",
                mono=mono,
            ),
            show_usage(
                [
                    "@ssm",
                    "get",
                    "path=<path>",
                ],
                "get <path>/sample.env secrets.",
                mono=mono,
            ),
        ]
    )


def help_put(
    tokens: List[str],
    mono: bool,
) -> str:
    return "\n".join(
        [
            show_usage(
                [
                    "@ssm",
                    "put",
                    "<secret-name>",
                    "<secret-value>",
                ]
                + ["[--description <description>]"],
                "put <secret-name> = <secret-value>.",
                mono=mono,
            ),
            show_usage(
                [
                    "@ssm",
                    "put",
                    "plugin=<plugin-name>",
                ],
                "put <plugin-name> secrets.",
                mono=mono,
            ),
            show_usage(
                [
                    "@ssm",
                    "put",
                    "repo=<repo-name>",
                ],
                "put <repo-name> secrets.",
                mono=mono,
            ),
        ]
    )


def help_rm(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@ssm",
            "rm",
            "<secret-name>",
        ],
        "rm <secret-name>.",
        mono=mono,
    )


help_functions = {
    "get": help_get,
    "put": help_put,
    "rm": help_rm,
}



from typing import List

from blue_options.terminal import show_usage, xtra


def help_publish(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("download,", mono=mono),
            "extension=png+geojson,push",
        ]
    )

    return show_usage(
        [
            "@assets",
            "publish",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "<object-name -> assets.",
        mono=mono,
    )


help_functions = {
    "publish": help_publish,
}



import boto3
from typing import Any, Tuple
from botocore.exceptions import ClientError

from blueness import module
from blue_objects.env import ABCLI_AWS_REGION

from abcli import NAME
from abcli import env
from abcli.logger import logger

NAME = module.name(__file__, NAME)


def get_secret(secret_name: str) -> Tuple[bool, str]:
    client = boto3.client(
        service_name="secretsmanager",
        region_name=ABCLI_AWS_REGION,
    )

    try:
        get_secret_value_response = client.get_secret_value(
            SecretId=secret_name,
        )
    except ClientError as e:
        if e.response["Error"]["Code"] == "ResourceNotFoundException":
            # If the secret does not exist
            return True, ""
        return False, f"error:{e}"
    except Exception as e:
        return False, f"error:{e}"

    return True, get_secret_value_response.get(
        "SecretString",
        "unknown",
    )


def put_secret(
    secret_name: str,
    secret_value: str,
    description: str = "",
) -> Tuple[bool, Any]:
    if not description:
        description = f"secret managed by {NAME}.put_secret()"

    client = boto3.client(
        "secretsmanager",
        region_name=ABCLI_AWS_REGION,
    )

    try:
        # Check if the secret already exists
        _ = client.describe_secret(SecretId=secret_name)
        # If secret exists, update it
        response = client.update_secret(
            SecretId=secret_name,
            SecretString=secret_value,
            Description=description,
        )
        logger.info(f"{NAME}.put_secret: {secret_name} updated.")
    except ClientError as e:
        if e.response["Error"]["Code"] == "ResourceNotFoundException":
            # If the secret does not exist, create it
            response = client.create_secret(
                Name=secret_name,
                SecretString=secret_value,
                Description=description,
            )
            logger.info(f"{NAME}.put_secret: {secret_name} updated.")
        else:
            logger.error(f"{NAME}.put_secret({secret_name}): AWS Client Error: {e}")
            return False, {}
    except Exception as e:
        logger.error(f"{NAME}.put_secret({secret_name}): error: {e}.")
        return False, {}

    return True, response


def rm_secret(secret_name: str) -> Tuple[bool, Any]:
    client = boto3.client(
        "secretsmanager",
        region_name=ABCLI_AWS_REGION,
    )

    try:
        response = client.delete_secret(
            SecretId=secret_name,
            ForceDeleteWithoutRecovery=True,
        )
        logger.info(f"{NAME}.rm_secret: {secret_name} deleted.")
    except ClientError as e:
        logger.error(f"{NAME}.rm_secret({secret_name}): AWS Client Error: {e}")
        return False, {}
    except Exception as e:
        logger.error(f"{NAME}.rm_secret({secret_name}): error: {e}.")
        return False, {}

    return True, response



from typing import List

from blue_options.terminal import show_usage, xtra


def help_init(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "clear,~terraform"

    return show_usage(
        [
            "@init",
            "[<plugin-name> | all | clear] ",
            f"[{options}]",
        ],
        "init [<plugin-name>].",
        mono=mono,
    )



from typing import List, Tuple

import numpy as np
import platform

from blueness import module
from blue_options import host, string
from blue_options.logger import crash_report
from blue_options.env import abcli_wifi_ssid

from abcli import NAME, fullname
from abcli.logger import logger

NAME = module.name(__file__, NAME)


def lxde(_):
    return terraform(
        ["/etc/xdg/lxsession/LXDE/autostart"],
        [
            "@bash /home/pi/git/awesome-bash-cli/abcli/.abcli/abcli.sh - abcli session start"
        ],
    )


def poster(filename: str) -> bool:
    from blue_objects.graphics.text import render_text
    from blue_objects.graphics.frame import add_frame
    from blue_objects.graphics import screen
    from blue_objects import file

    logger.debug("{}.poster({})".format(NAME, filename))

    image = add_frame(
        np.concatenate(
            [
                render_text(
                    centered=True,
                    image_width=screen.get_size()[1],
                    text=line,
                    thickness=4,
                )
                for line in signature()
            ],
            axis=0,
        ),
        32,
    )

    return image if filename is None else file.save_image(filename, image)


def mac(user):
    return terraform(
        ["/Users/{}/.bash_profile".format(user)],
        ["source ~/git/awesome-bash-cli/abcli/.abcli/abcli.sh"],
    )


def rpi(_, is_headless=False):
    success = terraform(
        ["/home/pi/.bashrc"],
        [
            "source /home/pi/git/awesome-bash-cli/abcli/.abcli/abcli.sh{}".format(
                "  - abcli session start" if is_headless else ""
            )
        ],
    )

    if not is_headless:
        if not terraform(
            ["/etc/xdg/lxsession/LXDE-pi/autostart"],
            [
                "@bash /home/pi/git/awesome-bash-cli/abcli/.abcli/abcli.sh  - abcli session start"
            ],
        ):
            success = False

    return success


def load_text_file(
    filename: str,
) -> Tuple[bool, List[str]]:
    try:
        with open(filename, "r") as fp:
            text = fp.read()
        text = text.split("\n")

        return True, text
    except:
        return False, []


def save_text__file_if_different(
    filename: str,
    text: List[str],
) -> bool:
    _, current_text = load_text_file(filename)
    if "|".join([line for line in current_text if line]) == "|".join(
        [line for line in text if line]
    ):
        return True

    try:
        with open(filename, "w") as fp:
            fp.writelines([string + "\n" for string in text])

        logger.info(f"updated {filename} ...")
        return True
    except Exception as e:
        crash_report(e)
        return False


def terraform(
    filenames: List[str],
    commands: List[str],
) -> bool:
    success = True
    for filename, command in zip(filenames, commands):
        success_, content = load_text_file(filename)
        if not success_:
            success = False
            continue

        content_updated = [
            string
            for string in content
            if ("git/awesome-bash-cli" not in string) and string
        ] + [command]

        if not save_text__file_if_different(
            filename,
            content_updated,
        ):
            success = False

    return success


def signature() -> List[str]:
    return [
        fullname(),
        host.get_name(),
        " | ".join(
            host.tensor_processing_signature()
            + [
                f"Python {platform.python_version()}",
                f"{platform.system()} {platform.release()}",
            ]
        ),
        " | ".join(
            [
                string.pretty_date(include_time=False),
                string.pretty_date(
                    include_date=False,
                    include_zone=True,
                ),
            ]
            + ([abcli_wifi_ssid] if abcli_wifi_ssid else [])
        ),
    ]


def ubuntu(user):
    return terraform(
        ["/home/{}/.bashrc".format(user)],
        ["source /home/{}/git/awesome-bash-cli/abcli/.abcli/abcli.sh".format(user)],
    )
